#include "ocsinv.h"

#ifdef HAVE_SYS_SYSTEMINFO_H
#include <sys/systeminfo.h>
#endif

#include <string.h>
#include <unistd.h>


/*
 * lookup the given command in $PATH, and return the complete path (or NULL)
 * WARNING: static buffer is returned;
 */

static char*
lookup_in_path(const char* cmd)
{
	static char ret[PATH_MAX];
	char* env_path, *env_path0;
	char* tmp;

	env_path= strdup ( getenv("PATH"));
	if (!env_path) return NULL; 

	env_path0 = env_path;
	tmp = strtok (env_path, ":");
	while (tmp)
	{
		snprintf (ret, PATH_MAX, "%s/%s", tmp, cmd);
		if (access (ret, R_OK) == 0)
		{
			free (env_path0);
			return ret;
		}
		tmp = strtok (NULL, ":");
	}
	free (env_path0);
	return NULL;
}


#ifdef SOLARIS
void solaris_info_bios(xmlNodePtr node)
{
	xmlNodePtr bios_node;
	char buf[256];
	char* sneep_path;

	bios_node = xmlNewChild (node, NULL, "BIOS", NULL);
	assert (bios_node !=NULL);

	sysinfo (SI_PLATFORM, buf, sizeof(buf));
	xmlNewTextChild (bios_node, NULL, "SMODEL", buf);

	sysinfo (SI_HW_PROVIDER, buf, sizeof(buf));
	xmlNewTextChild (bios_node, NULL, "SMANUFACTURER", buf);  

	if ((sneep_path=lookup_in_path ("sneep")))
	{
		FILE* sneep;

		sneep = popen (sneep_path, "r");
		if (sneep)
		{
			fgets(buf, 256, sneep);
			pclose (sneep);
			if (strlen(buf) >0) buf[strlen(buf)-1]='\0'; 
			xmlNewTextChild (bios_node, NULL, "SSN", buf);  
		}
	}

}
#endif /* SOLARIS */


#ifdef BSD
#include <sys/utsname.h>

void bsd_info_bios(xmlNodePtr node)
{
	struct utsname info;
	xmlNodePtr bios_node;

	if (uname (&info) != 0)
	{
		perror("uname()");
		return;
	}

	bios_node = xmlNewChild (node, NULL, "BIOS", NULL);
	assert (bios_node != NULL);

	xmlNewTextChild(bios_node, NULL, "SMODEL", info.machine);
	xmlNewTextChild(bios_node, NULL, "SMANUFACTURER", info.sysname);

/* TODO: check other BIOS nodes */
}
#endif /* BSD */

