#include "ocsinv.h"

#include <stdio.h>
#include <sys/types.h>
#ifdef SOLARIS
#include <sys/mnttab.h>
#endif
#include <sys/types.h>

#if defined(HAVE_SYS_STATVFS_H)
#include <sys/statvfs.h>
#elif defined(HAVE_STATFS)
#include <sys/param.h>
#include <sys/mount.h>
#endif 
#include <string.h>
#include "ocsinv.h"


/* do not report drive info for these following volumes types: */
static const char* skip_vol_types[] =  {
	 "nfs", 
	 "proc", 
	 "automount",
	 "autofs",
	 "tmpfs",
	 "fd",
	 "mntfs",
	 "mvfs", /* Clearcase */
	 "devfs",
	 "ctfs",
	 "objfs",
	 "swap",
	 "cd9660",
	 NULL
	 };

static int skip_drive (const char* type)
{
	int idx;
	const char* tmp;

	idx=0;
	while ((tmp=skip_vol_types[idx++]))
		if (strcasecmp(type,tmp)==0) 
			return 1;

  return 0;
}

#ifdef SOLARIS
void solaris_info_drives (xmlNodePtr node)
{

	FILE* fp;
	struct mnttab mp;
	struct statvfs sp;
	xmlNodePtr drive_node;
	int   drive_idx;
	char  buf[32];

	drive_idx = 0;

	fp = fopen (MNTTAB, "r");
	if (!fp)
	{
		perror(MNTTAB);
		return ;
	}

	while (getmntent(fp, &mp) == 0)
	{
		if (statvfs (mp.mnt_mountp, &sp) != 0)
			perror(mp.mnt_mountp);
		else
		{

			if (skip_drive (mp.mnt_fstype))
				continue;

			snprintf(buf, 32, "DRIVES");
			drive_node = xmlNewChild (node, NULL, buf, NULL);
			if (!drive_node) continue;

			xmlNewTextChild (drive_node, NULL, "FILESYSTEM", mp.mnt_fstype);
			xmlNewTextChild (drive_node, NULL, "TYPE", mp.mnt_special);
			xmlNewTextChild (drive_node, NULL, "LETTER", mp.mnt_mountp);

			snprintf(buf, 32, "%.0f", (float) sp.f_blocks * (float)  sp.f_frsize / (1024*1024));
			xmlNewTextChild (drive_node, NULL, "TOTAL", buf);

			snprintf(buf, 32, "%.0f", (float) sp.f_bfree* (float)  sp.f_frsize / (1024*1024));
			xmlNewTextChild (drive_node, NULL, "FREE", buf);

			drive_idx++;
		}
	}
	fclose (fp);
}
#endif /* SOLARIS */


#ifdef BSD
#include <fstab.h>
void bsd_info_drives(xmlNodePtr node)
{
	struct fstab* fs;
#if defined(HAVE_STATVFS)
	struct statvfs fss;
#elif defined(HAVE_STATFS)
	struct statfs  fss;
#endif
	char buf[16];
	xmlNodePtr drive_node;

	setfsent();

	while ((fs = getfsent()) != NULL)
	{
		if (skip_drive (fs->fs_vfstype)) 
			continue;

		drive_node = xmlNewChild (node, NULL, "DRIVES", NULL);
		assert(drive_node != NULL);

		xmlNewTextChild (drive_node, NULL, "FILESYSTEM", fs->fs_spec);
		xmlNewTextChild (drive_node, NULL, "TYPE",       fs->fs_vfstype);
		xmlNewTextChild (drive_node, NULL, "LETTER",     fs->fs_file);

#if defined(HAVE_STATVFS)
		if (statvfs (fs->fs_file, &fss) != 0)
			perror("statvfs()");
#elif defined(HAVE_STATFS)
                if (statfs (fs->fs_file, &fss) != 0)
			perror("statfs()");
#define f_frsize f_blocks

#endif
		else
		{
			snprintf(buf, 16, "%0.f", (float) fss.f_blocks * (float) fss.f_frsize / (1024*1024));
			xmlNewTextChild (drive_node, NULL, "TOTAL", buf);

			snprintf(buf, 16, "%0.f", (float) fss.f_bfree * (float) fss.f_frsize / (1024*1024));
			xmlNewTextChild (drive_node, NULL, "FREE", buf);
		}
#undef f_frsize	
		

	}

}
#endif /* BSD */

