#include "ocsinv.h"

#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <sys/stat.h>


#ifdef SOLARIS
#include <limits.h>
#include <sys/swap.h>
#define PAGE_SIZE 512

void solaris_info_memory(xmlNodePtr node)
{
	unsigned long long memtotal;
	unsigned long long swptotal;
	struct swaptable *swptbl;
	struct swapent swpent;
	int n_swp;
	int idx;
	char buf[8];


/* main (real) memory statistics */
	memtotal = (unsigned long long) sysconf (_SC_PAGE_SIZE)  * (unsigned long long) sysconf(_SC_PHYS_PAGES);
	snprintf(buf, 8, "%.0f", (float)memtotal / (1024*1024));
	xmlNewTextChild( node, NULL, "MEMORY", buf);


/* swap memory statistics */

	swptotal=0;
	n_swp = swapctl(SC_GETNSWP, NULL);
	if (n_swp  > 0)
	{

		swptbl = (struct swaptable*) malloc (n_swp * (sizeof(struct swaptable)));

		for (idx=0; idx < n_swp; idx++)
			swptbl->swt_ent[idx].ste_path = (char*) malloc (PATH_MAX);

		swptbl->swt_n = n_swp + 1;
		if (swapctl(SC_LIST, swptbl) <0)
		{
			perror ("swapctl()");
		}
		
		else

		for (idx=0; idx < n_swp; idx++)
		{
			swpent = swptbl->swt_ent[idx];
			swptotal += (unsigned long long) swpent.ste_length;
		}
	}

	snprintf(buf, 8, "%0.f", ((double)swptotal / (1024*1024)) * PAGE_SIZE);
	xmlNewTextChild (node, NULL, "SWAP", buf);

}
#endif /* SOLARIS */


#ifdef BSD
#include <fcntl.h>
#include <kvm.h>
#include <sys/param.h>
#include <sys/sysctl.h>

void bsd_info_memory(xmlNodePtr node)
{
	kvm_t*	kd;
#ifdef HAVE_KVM_SWAP
	struct kvm_swap si[1];
#endif
	char buf[8];
	int mib[2];
	int phymem, phymem_len;

#if OPENBSD
	kd = kvm_open (NULL, NULL, NULL, O_RDONLY | KVM_NO_FILES , ""); 
#else
	kd = kvm_open (NULL, "/dev/null", NULL, O_RDONLY, "");
#endif
	if (!kd)
		return; /* kvm_open() should have displayed the error message. */

#ifdef HAVE_KVM_SWAP
	if ( kvm_getswapinfo (kd, si, 1, 0) != -1)
	{
		snprintf(buf, 8, "%0.f", ((double) si[0].ksw_total / (1024*1024)) * getpagesize());
		xmlNewTextChild (node, NULL, "SWAP", buf);
	}
        kvm_close (kd);
#endif

	mib[0] = CTL_HW;
	mib[1] = HW_PHYSMEM;
	
	phymem_len = sizeof (phymem);
	if (sysctl (mib, 2, &phymem, &phymem_len, NULL, 0) == 0)
	{
		snprintf(buf, 8, "%0.f", (double) phymem / (1024*1024));
		xmlNewTextChild (node, NULL, "MEMORY", buf);
	}
	else
		perror("sysctl(CTL_HW.HW_PHYSMEM)");
	
}
#endif /* BSD */

