#include "ocsinv.h"

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <sys/sockio.h>
#include <string.h>


#ifdef SOLARIS
void solaris_info_network (xmlNodePtr node)
{

	int s;
	int n_ifs;
	struct lifnum lifnum;
	struct lifconf lifconf;
	struct lifreq *lifreq;
	int n;
	int sock_f;
	int if_flags;
	char buf[256];
	xmlNodePtr net_node;
	struct sockaddr_in ip_addr;
	struct sockaddr_in net_addr;

	s = socket (AF_INET,SOCK_DGRAM,IPPROTO_UDP);
	if (s < 0)
	{
		perror("socket()");
		return ;
	}

/* get number of interfaces */

	lifnum.lifn_family = AF_UNSPEC; 
	lifnum.lifn_flags = 0;
	lifnum.lifn_count = 0;
	if ( ioctl (s, SIOCGLIFNUM, &lifnum) !=0)
	{
		perror("ioctl(SIOCGLIFNUM)");
		return ;
	}
	n_ifs = lifnum.lifn_count;

/* retrieve all interfaces info */

	lifconf.lifc_family = AF_UNSPEC;
	lifconf.lifc_flags  = 0;
	lifconf.lifc_len    = sizeof (struct lifreq) * n_ifs;
	lifconf.lifc_req     = (struct lifreq*) malloc (lifconf.lifc_len );

	if ( ioctl (s, SIOCGLIFCONF, &lifconf) != 0)
	{
		perror("ioctl(SIOCGLIFCONF)");
		return ;
	}

	for (n=0; n < n_ifs; n++)
	{

		lifreq = &lifconf.lifc_req[n];

		if ( ioctl (s, SIOCGLIFFLAGS, lifreq) != 0)
		{
			perror("SIOCGLIFFLAGS");
			continue;
		}

		if_flags = lifreq->lifr_flags;
		if ( ! (if_flags  & IFF_IPV4)) continue; /* not IPv4 if */
		if ((if_flags & IFF_LOOPBACK)) continue; /* skip loopback iface */

		snprintf(buf, 256, "NETWORKS");
		net_node = xmlNewChild (node, NULL, buf, NULL);
		if (!net_node) continue;

		xmlNewTextChild (net_node, NULL, "DESCRIPTION", lifreq->lifr_name);

		if ( ioctl (s, SIOCGLIFADDR, lifreq) != 0)
		{
			perror("SIOCGLIFADDR");
			continue;
		}

		sock_f = lifreq->lifr_addr.ss_family;
		if (sock_f == AF_INET)
		{
			struct arpreq  arp_r;
			xmlNewTextChild (net_node, NULL, "IPADDRESS", inet_ntoa( ((struct sockaddr_in*) &lifreq->lifr_addr)->sin_addr));
			memcpy (&ip_addr, &lifreq->lifr_addr, sizeof (struct sockaddr_in));

			/* get MAC addr */
			memcpy (& (arp_r.arp_pa), &(lifreq->lifr_addr), sizeof (struct sockaddr));
			if (ioctl (s, SIOCGARP, &arp_r) == 0)
			{
				unsigned char* p=(unsigned char *)&(arp_r.arp_ha.sa_data);
				snprintf(buf, sizeof (buf), "%02X:%02X:%02X:%02X:%02X:%02X",
										                p[0],p[1],p[2],p[3],p[4],p[5]);

				xmlNewTextChild (net_node, NULL, "MACADDR", buf );	
			}
			else
				xmlNewTextChild(net_node, NULL, "MACADDR", "ff:ff:ff:ff:ff:ff");
		}
		if (ioctl (s, SIOCGLIFBRDADDR, lifreq) == 0)
			xmlNewTextChild (net_node, NULL, "BROADCAST", inet_ntoa( ((struct sockaddr_in*) &lifreq->lifr_addr)->sin_addr));

		if (ioctl (s, SIOCGLIFNETMASK, lifreq) == 0)
		{
			xmlNewTextChild (net_node, NULL, "IPMASK", inet_ntoa( ((struct sockaddr_in*) &lifreq->lifr_addr)->sin_addr));
			net_addr.sin_addr.s_addr =  ip_addr.sin_addr.s_addr & ((struct sockaddr_in*) &lifreq->lifr_addr)->sin_addr.s_addr;

			xmlNewTextChild (net_node, NULL, "IPSUBNET", inet_ntoa (net_addr.sin_addr));
		}
			
		
		xmlNewTextChild (net_node, NULL, "TYPE",   (if_flags & IFF_LOOPBACK) ? "loopback": "ethernet");
		xmlNewTextChild (net_node, NULL, "STATUS", (if_flags & IFF_UP) ? "up" : "down");
	}
}
#endif /* SOLARIS */



#ifdef BSD
#include <sys/types.h>
#include <sys/socket.h>
#include <ifaddrs.h>
#include <net/if_dl.h>

void bsd_info_network(xmlNodePtr node)
{
	xmlNodePtr net_node;
	struct ifaddrs* ifa, *ifa2, *if0;
	struct in_addr ip_subnet;
	char buf[6*3+1];

	if (getifaddrs(&if0) != 0)
	{
		perror("getifaddrs()");
		return;
	}

	for ( ifa=if0; ifa; ifa=ifa->ifa_next)
	{
		if (! (ifa->ifa_flags & IFF_UP)) continue; /* interface down */
		if (   ifa->ifa_flags & IFF_LOOPBACK) continue; /* loopback */
		if ( ifa->ifa_addr->sa_family != AF_INET) continue; /* not IPv4 */
			
		net_node = xmlNewChild (node, NULL, "NETWORKS", NULL);			
		assert (net_node != NULL);
		
		xmlNewTextChild (net_node, NULL, "DESCRIPTION", ifa->ifa_name);
		xmlNewTextChild (net_node, NULL, "IPADDRESS", 
				inet_ntoa ( ((struct sockaddr_in*) ifa->ifa_addr)->sin_addr));
		xmlNewTextChild (net_node, NULL, "BROADCAST",  
				inet_ntoa ( ((struct sockaddr_in*) ifa->ifa_broadaddr)->sin_addr));
		xmlNewTextChild (net_node, NULL, "IPMASK",  
				inet_ntoa ( ((struct sockaddr_in*) ifa->ifa_netmask)->sin_addr));

		ip_subnet.s_addr =  (((struct sockaddr_in*) ifa->ifa_addr)->sin_addr.s_addr &
		                        (((struct sockaddr_in*) ifa->ifa_netmask)->sin_addr.s_addr) );
					
                xmlNewTextChild (net_node, NULL, "IPSUBNET", inet_ntoa (ip_subnet));

		/* lookup link-level address of this interface: rescan interfaces list */;
		for (ifa2=if0; ifa2; ifa2=ifa2->ifa_next)
		{
			if (ifa2->ifa_addr->sa_family != AF_LINK) continue;
			if (strcmp (ifa2->ifa_name, ifa->ifa_name) != 0 ) continue; 

			/* found it ! */
			unsigned char* p=(unsigned char *) ( ( (struct sockaddr_dl*) ifa2->ifa_addr)->sdl_data);
			p+= strlen (ifa2->ifa_name); /* on BSD, the interface name is prefixed to this hw address. */
			snprintf(buf, sizeof (buf), "%02X:%02X:%02X:%02X:%02X:%02X",
					p[0],p[1],p[2],p[3],p[4],p[5]);
                        xmlNewTextChild (net_node, NULL, "MACADDR", buf );

			/* link speed */
			if ((((struct if_data*) ifa2->ifa_data)->ifi_baudrate) == 0)
				snprintf( buf, sizeof (buf), "auto-select"); /* TODO: check this assertion.*/
			else	 
				snprintf (buf, sizeof (buf), "%i", (int)(((struct if_data*) ifa2->ifa_data)->ifi_baudrate));
				    			
			xmlNewTextChild (net_node, NULL, "SPEED", buf);
			break;	
		}
	}
	
}
#endif /* BSD */


