#ifndef __OCSINV_H
#define __OCSINV_H

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <libxml/tree.h>

extern void info_content          (xmlNodePtr);

#ifdef SOLARIS
extern void solaris_info_memory   (xmlNodePtr);
extern void solaris_info_drives   (xmlNodePtr);
extern void solaris_info_os       (xmlNodePtr);
extern void solaris_info_network  (xmlNodePtr);
extern void solaris_info_storage  (xmlNodePtr);
extern void solaris_info_bios     (xmlNodePtr);
extern void solaris_info_software (xmlNodePtr);

#define info_memory(x)   solaris_info_memory(x)
#define info_drives(x)   solaris_info_drives(x)
#define info_os(x)       solaris_info_os(x)
#define info_network(x)  solaris_info_network(x)
#define info_storage(x)  solaris_info_storage(x)
#define info_bios(x)     solaris_info_bios(x)
#define info_software(x) solaris_info_software(x)
#endif  /* SOLARIS */


#ifdef BSD
extern void bsd_info_memory   (xmlNodePtr);
extern void bsd_info_drives   (xmlNodePtr);
extern void bsd_info_os       (xmlNodePtr);
extern void bsd_info_network  (xmlNodePtr);
extern void bsd_info_storage  (xmlNodePtr);
extern void bsd_info_bios     (xmlNodePtr);
extern void bsd_info_software (xmlNodePtr);

#define info_memory(x)   bsd_info_memory(x)
#define info_drives(x)   bsd_info_drives(x)
#define info_os(x)       bsd_info_os(x)
#define info_network(x)  bsd_info_network(x)
#define info_storage(x)  bsd_info_storage(x)
#define info_bios(x)     bsd_info_bios(x)
#define info_software(x) bsd_info_software(x)
#endif /* BSD */


#define AGENT_PLATFORM   PLATFORM
#define AGENT_VERSION    PACKAGE_VERSION

#define assert(x) { \
	if ( ! (x)) { \
		fprintf(stderr, "** assertion failed: %s\n", #x); \
		abort();\
	} \
} while (0)

#define LOG(fmt, ...) { \
  fprintf(stdout, fmt, __VA_ARGS__); \
  fprintf(stdout,"\n"); \
  fflush (stdout);\
} while (0)
  
  
#define DEBUG(fmt, ...) {\
	if (debug) { fprintf(stdout, "(DEBUG) "); LOG(fmt, __VA_ARGS__);} \
}  while (0)
  
  

/* main.c */
extern int debug;
extern char *deviceid_dir;
extern char *myname;
/* content.c */
extern char* get_device_id();

/* ocs.c */

#include <curl/curl.h>

typedef struct OCS_t
{
	char* server;
  CURL* curl;
} OCS_t;

OCS_t*    OCS_new(const char*);
xmlDocPtr OCS_request(OCS_t*, const xmlDocPtr);
int       OCS_update_agent(OCS_t*, int);
void      OCS_free (OCS_t*);

xmlDocPtr OCS_new_doc (const char*);

int       OCS_check_inventory(OCS_t*);
int       OCS_check_agent_update (OCS_t*);
int       OCS_send_inventory (OCS_t*, const xmlDocPtr);
int       OCS_update_agent (OCS_t*, int);


/* connection & operations timeout */
#define OCS_CONNECT_TIMEOUT 10L
#define OCS_TIMEOUT        300L

#define XML_VERSION   "1.0"

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 256
#endif

#endif /* ! __OCSINV_H */
