#include "ocsinv.h"

#include <netdb.h>
#include <unistd.h>

#ifdef SOLARIS
#include <sys/systeminfo.h>
#include <kstat.h>
#endif

#include <inttypes.h>
#include <time.h>
#include <string.h>

#ifdef SOLARIS

/* Solaris 8 misses this macro */
 
#ifndef KSTAT_NAMED_STR_PTR 
# define KSTAT_NAMED_STR_PTR(x) ( (x)->value.c) 
#endif 

void solaris_info_os (xmlNodePtr node)
{
	char hostname[MAXHOSTNAMELEN];
	char buf[256];
	kstat_ctl_t *kc;
	kstat_t* ks;
	kstat_named_t* knp;
	int ncpus;

	if (myname != NULL)
		xmlNewTextChild(node, NULL, "NAME", myname);
	else
		if (gethostname(hostname, MAXHOSTNAMELEN) == 0)
			xmlNewTextChild(node, NULL, "NAME", hostname);

	if (sysinfo (SI_RELEASE, buf, sizeof(buf)) > 0)
		xmlNewTextChild (node, NULL, "OSVERSION", buf);

	if (sysinfo (SI_SYSNAME, buf, sizeof(buf)) > 0)
		xmlNewTextChild (node, NULL, "OSNAME", buf);


/*
 *	NOTE: OCS does not support different processor models & speeds
 *  on the same machine.
 */

	kc = kstat_open();
	if (!kc)
	{
		perror("kstat_open()");
		return;
	}

	ncpus=0;

	for (ks=kc->kc_chain; ks; ks=ks->ks_next)
	{

		if  (strcasecmp("cpu_info", ks->ks_module) != 0)  continue;
		ncpus++;

		if (ncpus==1)
		{
			kstat_read (kc, ks, NULL) ;

			if ( (knp = kstat_data_lookup (ks, "brand")))
				xmlNewTextChild( node, NULL, "PROCESSORT", KSTAT_NAMED_STR_PTR (knp));
			else if ((knp = kstat_data_lookup (ks, "implementation")))
				xmlNewTextChild( node, NULL, "PROCESSORT", KSTAT_NAMED_STR_PTR (knp));
			else if ((knp = kstat_data_lookup (ks, "cpu_type")))
				xmlNewTextChild( node, NULL, "PROCESSORT", KSTAT_NAMED_STR_PTR (knp));

			knp = kstat_data_lookup (ks, "clock_MHz");
			if (knp)
				switch (knp->data_type)
				{
					case KSTAT_DATA_INT32:  
						snprintf(buf, sizeof(buf), "%" PRId32,knp->value.i32);  
						break;  
					case KSTAT_DATA_UINT32:  
						snprintf(buf, sizeof(buf), "%" PRIu32,knp->value.ui32);  
						break;  
					case KSTAT_DATA_INT64:  
						snprintf(buf, sizeof(buf), "%" PRId64,knp->value.i64);  
						break;  
					case KSTAT_DATA_UINT64:  
						snprintf(buf, sizeof(buf), "%" PRIu64,knp->value.ui64);  
				}

			xmlNewTextChild (node, NULL, "PROCESSORS", buf);
		}
	}

	kstat_close (kc);
	snprintf(buf, sizeof(buf), "%i", ncpus);
	xmlNewTextChild (node, NULL, "PROCESSORN", buf);
}		
#endif /* SOLARIS */

	
#ifdef BSD
#include <sys/utsname.h>
#include <sys/sysctl.h>

void bsd_info_os(xmlNodePtr node)
{
	char hostname[MAXHOSTNAMELEN], domainname[MAXHOSTNAMELEN];
	char buf[256];
	int ncpus;
	struct utsname unam;
	int mib[2];
	int tmpi;
	char* tmps;
	int tmplen;

	if (myname != NULL)
		xmlNewTextChild(node, NULL, "NAME", myname);
	else
		if (gethostname(hostname, MAXHOSTNAMELEN) == 0)
			xmlNewTextChild(node, NULL, "NAME", hostname);
	if (getdomainname (domainname, MAXHOSTNAMELEN) == 0)
		xmlNewTextChild (node, NULL, "WORKGROUP", domainname);
			
	if ( uname (&unam) == 0)
	{
		xmlNewTextChild (node, NULL, "OSVERSION", unam.release);
		xmlNewTextChild (node, NULL, "OSNAME", unam.sysname);
	}

	ncpus=0;
	tmplen = sizeof (tmpi);
	mib[0] = CTL_HW; mib[1] = HW_NCPU;
	if (sysctl ( mib, 2, &tmpi, &tmplen, NULL, 0) ==0)
	{
		ncpus =tmpi;
		snprintf(buf, sizeof(buf)-1, "%i", ncpus);
		xmlNewTextChild (node, NULL, "PROCESSORN", buf);
		
		/* get CPU speed & model of the first CPU in system. We assume that other CPUs are equivalent.
		 * (at least, OCS assumes this .)
		 */
		 
		tmps = buf;
		tmplen = sizeof(buf);
	
#ifdef FREEBSD 
		if (sysctlbyname ("dev.cpu.0.%desc", tmps, &tmplen, NULL, 0) == 0)
		{
			tmps[tmplen]='\0';
			xmlNewTextChild (node, NULL, "PROCESSORT",buf);
		}

		tmplen = sizeof (tmpi);
		if (sysctlbyname ("dev.cpu.0.freq", &tmpi, &tmplen, NULL, 0) == 0)
		{
			snprintf (buf, sizeof(buf)-1, "%i", tmpi);
			xmlNewTextChild (node, NULL, "PROCESSORS", buf);
		}
#elif defined(OPENBSD)

		mib[0] = CTL_HW;
		mib[1] = HW_CPUSPEED;
		tmplen= sizeof(tmpi);

		if ( sysctl (mib, 2, &tmpi, &tmplen, NULL, 0) == 0)
		{
			snprintf (buf, sizeof(buf)-1, "%i", tmpi);
			xmlNewTextChild (node, NULL, "PROCESSORS", buf);
		}

		/* TODO: check PROCESSORT ? */

		mib[1] = HW_MODEL;
		tmplen = sizeof(buf);
		tmps   = buf;

		if ( sysctl (mib, 2, tmps, &tmplen, NULL, 0) == 0)
		{
			 tmps[tmplen]='\0';
			 xmlNewTextChild (node, NULL, "PROCESSORT", tmps);
		}

#endif /* OPENBSD */

	}

}
#endif /* BSD */
