#include "ocsinv.h"

#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>

#ifdef SOLARIS
#define PKGINFO_ROOT "/var/sadm/pkg"
#define PKGINFO_FILE "pkginfo"

/*
 * report packages installed with pkgadd(1M)
 *
 */


static char* strsplit(char** in, char sep)
{
	char* p =*in;
	char* q =*in;

	while (*(p++))
	{
		if (*p==sep)
		{
			*p ='\0';
			*in=p;
			return q;
		}
	}

	return NULL;
}


void solaris_info_software(xmlNodePtr node)
{
	DIR* dir;
	struct dirent* di;
	char filename[FILENAME_MAX];
	struct stat st;
	FILE *f;
	int n;
	xmlNodePtr pkg_node;
	
	char buf[256];
	char* line;
	char* key;
		
	
	
	
	dir = opendir (PKGINFO_ROOT);
	if (!dir)
	{
		perror(PKGINFO_ROOT);
		return;
	}
	
	while ( (di=readdir(dir)))
	{
		if (di->d_name[0]=='.') continue;
		snprintf (filename, FILENAME_MAX, "%s/%s",  PKGINFO_ROOT, di->d_name);
		
		if (stat (filename, &st) != 0)
		{
			perror(filename);
			continue;
		}
		
		if ( ! S_ISDIR (st.st_mode)) continue;
		
		snprintf(filename, FILENAME_MAX, "%s/%s/%s", PKGINFO_ROOT, di->d_name, PKGINFO_FILE);
		if ( access (filename, R_OK) != 0)
		{
			perror(filename);
			continue;
		}
		
		f = fopen (filename, "r");
		if (!f)
		{
			perror(filename);
			continue;
		}
		
		n=0;
		
		pkg_node = xmlNewChild(node, NULL, "SOFTWARES", NULL);
		assert (pkg_node != NULL);
		
		while (!feof(f))
		{
			if ( ! (line= fgets(buf, sizeof(buf), f)))
				break;
			
			line[strlen(line)-1]='\0'; /* remove leading \n */
			key = strsplit(&line, '=');
			if (! key) continue;
			
			if (strcmp (key, "PKG")==0)
			{
				n++;
				xmlNewTextChild (pkg_node, NULL, "NAME", line+1);
			} else if (strcmp (key, "VENDOR")==0)
			{
				n++;
				xmlNewTextChild (pkg_node, NULL, "PUBLISHER", line+1);
			} else if (strcmp (key, "DESC")==0)
			{
				n++;
				xmlNewTextChild (pkg_node, NULL, "COMMENTS", line+1);
			} else if (strcmp (key, "VERSION")==0)
			{
				n++;
				xmlNewTextChild (pkg_node, NULL, "VERSION", line+1);
			}
			
			if (n==4) break;
		}
		
		fclose (f);
	
	}
	
	closedir (dir);
}
#endif /* SOLARIS */


#ifdef BSD
#include <regex.h>


#define PKG_PATH "/var/db/pkg"

void bsd_info_software(xmlNodePtr node)
{
	DIR* dir;
	struct dirent* di; 
	regex_t re_pkg;
	int     re_err;
	char    buf[FILENAME_MAX];
	regmatch_t re_matches[3];
	xmlNodePtr pkg_node;

	re_err = regcomp (&re_pkg, "(.+)\\-([0-9\\.\\_p]+)$", REG_EXTENDED);
	assert (re_err == 0);	

	dir = opendir (PKG_PATH);
	if (!dir)
	{
		perror(PKG_PATH);
		return;
	}

	while ( (di = readdir(dir)))
	{
		if (di->d_name[0] == '.') continue;
		if (!(di->d_type & DT_DIR)) continue;
		if ( strlen(di->d_name) > sizeof(buf)) continue; /* too long */

		if (regexec (&re_pkg, di->d_name, 3, re_matches,  0) != 0)
			continue; /* invalid package name */

		pkg_node = xmlNewChild (node, NULL, "SOFTWARES", NULL);
		
		memset (buf, 0, sizeof (buf));
		memcpy(buf, di->d_name + re_matches[1].rm_so, re_matches[1].rm_eo - re_matches[1].rm_so);
		xmlNewTextChild (pkg_node, NULL, "NAME", buf);

		memset (buf, 0, sizeof (buf));
		memcpy(buf, di->d_name + re_matches[2].rm_so, re_matches[2].rm_eo - re_matches[2].rm_so);
		xmlNewTextChild (pkg_node, NULL, "VERSION", buf);
		
	}
	closedir (dir);

	
}
#endif /* BSD */

