#include "ocsinv.h"

#include <unistd.h>
#ifdef SOLARIS
#include <sys/systeminfo.h>
#include <kstat.h>
#endif
#include <inttypes.h>
#include <time.h>
#include <string.h>


#ifdef SOLARIS
static void print_knp(char* buf, size_t buf_len, kstat_named_t* knp, long divider)
{

	memset(buf, 0, buf_len);

	switch (knp->data_type)
	{
		case KSTAT_DATA_INT32:
			snprintf(buf, buf_len, "%" PRId32,knp->value.i32 / (int32_t) divider);
			break;
		case KSTAT_DATA_UINT32:
			snprintf(buf, buf_len, "%" PRIu32,knp->value.ui32 / (uint32_t) divider);
			break;
		case KSTAT_DATA_INT64:
			snprintf(buf, buf_len, "%" PRId64,knp->value.i64 / (int64_t) divider);
			break;
		case KSTAT_DATA_UINT64:
			snprintf(buf, buf_len, "%" PRIu64,knp->value.ui64 / (uint64_t) divider);
			break;

		case KSTAT_DATA_CHAR:
			snprintf(buf, buf_len, "%.16s", knp->value.c);
			break;

		default:
			snprintf(buf, buf_len, "?");
	}
}

typedef struct storage_modules_t
{
	char *code;
	char *label;
} storage_modules_t;

static storage_modules_t modules[] = {
	{"sderr",    "SCSI or ATAPI disk (sd)"},
	{"ssderr",   "SCSI or ATAPI disk (ssd)"},
	{"sterr",    "SCSI or ATAPI Tape driver (st)"},
	{"daderror", "IDE disk (dad)"},
	{NULL, NULL}};

void solaris_info_storage (xmlNodePtr node)
{
	kstat_ctl_t* kc;
	kstat_t*     ks;
	kstat_named_t* knp;
	xmlNodePtr stg_node;
	char buf[256];

	kc = kstat_open();
	if (!kc)
	{
		perror("kstat_open()");
		return;
	}

	for (ks=kc->kc_chain; ks; ks=ks->ks_next)
	{
		storage_modules_t *mod = NULL;

		for (mod = modules; mod->code != NULL; mod++)
			if (strcasecmp(mod->code, ks->ks_module) == 0)
				break;

		if (mod->code == NULL)
			continue;

		stg_node = xmlNewChild (node, NULL, "STORAGES", NULL);
		assert (stg_node != NULL);     

		kstat_read (kc, ks, NULL) ;

		knp = kstat_data_lookup (ks, "Model");
		if (!knp)
			knp = kstat_data_lookup (ks, "Product");
		if (knp) {
			print_knp(buf, sizeof(buf), knp, 1);
			xmlNewTextChild (stg_node, NULL, "MODEL", buf);
		}

		knp = kstat_data_lookup (ks, "Vendor");
		if (knp) {
			print_knp(buf, sizeof(buf), knp, 1);
			xmlNewTextChild (stg_node, NULL, "MANUFACTURER", buf);
		}

		knp = kstat_data_lookup (ks, "Size");
		if (knp) {
			print_knp(buf, sizeof(buf), knp, 1024*1024);
			xmlNewTextChild (stg_node, NULL, "DISKSIZE", buf);
		}

		if (mod->label != NULL)
			xmlNewTextChild (stg_node, NULL, "TYPE", mod->label);
	}

	kstat_close (kc);
}
#endif /* SOLARIS */


#ifdef BSD
void bsd_info_storage(xmlNodePtr node)
{

}
#endif /* BSD */

